/**
 * @file	cf_thread.h
 * @author	myusgun <myusgun@gmail.com>
 * @version	0.1
 *
 * @remark 멀티스레드 및 뮤텍스 지원
 *
 * @example thread.c
 */
#ifndef __CF_THREAD_H__
#define __CF_THREAD_H__

#include "cf_base.h"

#ifdef _WIN32
# define CF_THREAD_RETURN	unsigned int	/**< 스레드 워커 함수 반환 형 */
# define CF_THREAD_CALL		__stdcall		/**< 스레드 워커 함수 호출 규약 */
#else // #ifdef _WIN32
# define CF_THREAD_RETURN	void *			/**< 스레드 워커 함수 반환 형 */
# define CF_THREAD_CALL						/**< 스레드 워커 함수 호출 규약 */
#endif // #ifdef _WIN32

/** 스레드 워커 함수 형태이며, CF_Thread_Function 형식 */
typedef CF_THREAD_RETURN  (CF_THREAD_CALL * CF_Thread_Function) (void *);

/** 스레드 아이디 */
typedef void *	CF_Thread;

/** 뮤텍스 아이디 */
typedef void *	CF_Mutex;

#ifdef __cplusplus
extern "C" {
#endif

CF_EXPORT int
CF_Thread_Create	(CF_Thread			* threadID,
					 CF_Thread_Function	callback,
					 void				* arg);

CF_EXPORT int
CF_Thread_Release	(CF_Thread * threadID);

CF_EXPORT int
CF_Thread_Join		(CF_Thread * threadID);

CF_EXPORT int
CF_Mutex_Create		(CF_Mutex * mutex);

CF_EXPORT int
CF_Mutex_Destory	(CF_Mutex * mutex);

CF_EXPORT int
CF_Mutex_Lock		(CF_Mutex * mutex);

CF_EXPORT int
CF_Mutex_Unlock		(CF_Mutex * mutex);

#ifdef __cplusplus
}
#endif

#endif // #ifndef __CF_THREAD_H__
